@@ -3,7 +3,7 @@
 #
 #  (C) Paul Evans, 2021-2023 -- leonerd@leonerd.org.uk
 
-package Syntax::Keyword::Match 0.12;
+package Syntax::Keyword::Match 0.13;
 
 use v5.14;
 use warnings;
@@ -30,6 +30,8 @@ C<Syntax::Keyword::Match> - a C<match/case> syntax for perl
       case(3) { say "It's three" }
       case(4), case(5)
               { say "It's four or five" }
+      case if($n < 10)
+              { say "It's less than ten" }
       default { say "It's something else" }
    }
 
@@ -58,30 +60,6 @@ warnings in the C<experimental> category, unless silenced.
 
 =cut
 
-my @EXPERIMENTAL = qw( dispatch );
-
-sub import
-{
-   shift;
-   my @syms = @_;
-
-   @syms or @syms = ( "match" );
-
-   my %syms = map { $_ => 1 } @syms;
-
-   $^H{"Syntax::Keyword::Match/match"}++ if delete $syms{match};
-
-   foreach ( @EXPERIMENTAL ) {
-      $^H{"Syntax::Keyword::Match/experimental($_)"}++ if delete $syms{":experimental($_)"};
-   }
-
-   if( delete $syms{":experimental"} ) {
-      $^H{"Syntax::Keyword::Match/experimental($_)"}++ for @EXPERIMENTAL;
-   }
-
-   croak "Unrecognised import symbols @{[ keys %syms ]}" if keys %syms;
-}
-
 =head1 KEYWORDS
 
 =head2 match
@@ -113,6 +91,15 @@ C<sub> or C<do> block. For example:
       }
    };
 
+If the controlling expression introduces a new variable, that variable will be
+visible within any of the C<case> blocks, and will go out of scope after the
+C<match> statement finishes. This may be useful for temporarily storing the
+result of a more complex expression.
+
+   match( my $x = some_function_call() : == ) {
+      case ...
+   }
+
 =head3 Comparison Operators
 
 The comparison operator must be either C<eq> (to compare cases as strings) or
@@ -180,6 +167,43 @@ is implemented with a custom operator that will entirely confuse modules like
 C<B::Deparse> or optree inspectors like coverage tools so is not selected by
 default, but can be enabled for extra performance in critical sections.
 
+=head2 case if
+
+   case if(EXPR) { STATEMENTS... }
+
+   case(VAL), case if(EXPR) { STATEMENTS... }
+
+A C<case> statement may also be written C<case if> with a boolean predicate
+expression in parentheses. This inserts a direct boolean test into the
+comparison logic, allowing for other logical tests that aren't easily
+expressed as uses of the comparison operator. As C<case if> is an alternative
+to a regular C<case>, they can be combined on a single code block if required.
+
+For example, when testing an inequality in a selection of numerical C<==>
+tests, or a single regexp test among some string C<eq> tests.
+
+   match( $num : == ) {
+      case(0)           { ... }
+      case(1), case(2)  { ... }
+      case if($num < 5) { ... }
+   }
+
+Z<>
+
+   match( $str : eq ) {
+      case("abc")           { ... }
+      case("def")           { ... }
+      case if($str =~ m/g/) { ... }
+   }
+
+By default the match value is not assigned into a variable that is visible
+to C<case if> expressions, but if needed a new lexical can be constructed by
+using a regular C<my> assignment.
+
+   match( my $v = some_expression() : eq ) {
+      case if($v =~ m/pattern/) { ... }
+   }
+
 =head2 default
 
 A C<default> statement must only appear inside the braces of a C<match>. If
@@ -247,6 +271,51 @@ and associated code are moved around into a different order. The syntax
 provided by this module notionally performs all of its tests in the order they
 are written in, and any changes of that order might cause a different result.
 
+=cut
+
+sub import
+{
+   my $pkg = shift;
+   my $caller = caller;
+
+   $pkg->import_into( $caller, @_ );
+}
+
+sub unimport
+{
+   my $pkg = shift;
+   my $caller = caller;
+
+   $pkg->unimport_into( $caller, @_ );
+}
+
+my @EXPERIMENTAL = qw( dispatch );
+
+sub import_into   { shift->apply( sub { $^H{ $_[0] }++ },      @_ ) }
+sub unimport_into { shift->apply( sub { delete $^H{ $_[0] } }, @_ ) }
+
+sub apply
+{
+   my $pkg = shift;
+   my ( $cb, $caller, @syms ) = @_;
+
+   @syms or @syms = ( "match" );
+
+   my %syms = map { $_ => 1 } @syms;
+
+   $cb->( "Syntax::Keyword::Match/match" ) if delete $syms{match};
+
+   foreach ( @EXPERIMENTAL ) {
+      $cb->( "Syntax::Keyword::Match/experimental($_)" ) if delete $syms{":experimental($_)"};
+   }
+
+   if( delete $syms{":experimental"} ) {
+      $cb->( "Syntax::Keyword::Match/experimental($_)" ) for @EXPERIMENTAL;
+   }
+
+   croak "Unrecognised import symbols @{[ keys %syms ]}" if keys %syms;
+}
+
 =head1 TODO
 
 This is clearly an early experimental work. There are many features to add,
